@@ -6,40 +6,29 @@
  * @ref License
  */
 
-/* LICENSE
- *
- * Copyright (c) 2008-2019 OpenShot Studios, LLC
- * <http://www.openshotstudios.com/>. This file is part of
- * OpenShot Library (libopenshot), an open-source project dedicated to
- * delivering high quality video editing and animation solutions to the
- * world. For more information visit <http://www.openshot.org/>.
- *
- * OpenShot Library (libopenshot) is free software: you can redistribute it
- * and/or modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * OpenShot Library (libopenshot) is distributed in the hope that it will be
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with OpenShot Library. If not, see <http://www.gnu.org/licenses/>.
- */
+// Copyright (c) 2008-2019 OpenShot Studios, LLC
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
 
 #include "Clip.h"
+
+#include "AudioResampler.h"
 #include "Exceptions.h"
 #include "FFmpegReader.h"
 #include "FrameMapper.h"
-#ifdef USE_IMAGEMAGICK
-	#include "ImageReader.h"
-	#include "TextReader.h"
-#endif
 #include "QtImageReader.h"
 #include "ChunkReader.h"
 #include "DummyReader.h"
 #include "Timeline.h"
+#include "ZmqLogger.h"
+
+#ifdef USE_IMAGEMAGICK
+    #include "MagickUtilities.h"
+    #include "ImageReader.h"
+    #include "TextReader.h"
+#endif
+
+#include <Qt>
 
 using namespace openshot;
 
@@ -50,7 +39,7 @@ void Clip::init_settings()
 	Position(0.0);
 	Layer(0);
 	Start(0.0);
-	End(0.0);
+	ClipBase::End(0.0);
 	gravity = GRAVITY_CENTER;
 	scale = SCALE_FIT;
 	anchor = ANCHOR_CANVAS;
@@ -158,7 +147,7 @@ Clip::Clip(ReaderBase* new_reader) : resampler(NULL), reader(new_reader), alloca
 
 	// Update duration and set parent
 	if (reader) {
-		End(reader->info.duration);
+		ClipBase::End(reader->info.duration);
 		reader->ParentClip(this);
 		// Init reader info struct
 		init_reader_settings();
@@ -217,7 +206,7 @@ Clip::Clip(std::string path) : resampler(NULL), reader(NULL), allocated_reader(N
 
 	// Update duration and set parent
 	if (reader) {
-		End(reader->info.duration);
+		ClipBase::End(reader->info.duration);
 		reader->ParentClip(this);
 		allocated_reader = reader;
 		// Init reader info struct
@@ -232,6 +221,7 @@ Clip::~Clip()
 	if (allocated_reader) {
 		delete allocated_reader;
 		allocated_reader = NULL;
+		reader = NULL;
 	}
 
 	// Close the resampler
@@ -278,14 +268,37 @@ void Clip::SetAttachedClip(Clip* clipObject){
 /// Set the current reader
 void Clip::Reader(ReaderBase* new_reader)
 {
+    // Delete previously allocated reader (if not related to new reader)
+    // FrameMappers that point to the same allocated reader are ignored
+    bool is_same_reader = false;
+    if (new_reader && allocated_reader) {
+        if (new_reader->Name() == "FrameMapper") {
+            // Determine if FrameMapper is pointing at the same allocated ready
+            FrameMapper* clip_mapped_reader = (FrameMapper*) new_reader;
+            if (allocated_reader == clip_mapped_reader->Reader()) {
+                is_same_reader = true;
+            }
+        }
+    }
+    // Clear existing allocated reader (if different)
+    if (allocated_reader && !is_same_reader) {
+        reader->Close();
+        allocated_reader->Close();
+        delete allocated_reader;
+        reader = NULL;
+        allocated_reader = NULL;
+    }
+
 	// set reader pointer
 	reader = new_reader;
 
 	// set parent
-	reader->ParentClip(this);
+	if (reader) {
+        reader->ParentClip(this);
 
-	// Init reader info struct
-	init_reader_settings();
+        // Init reader info struct
+        init_reader_settings();
+    }
 }
 
 /// Get the current reader
@@ -312,7 +325,7 @@ void Clip::Open()
 
 		// Set some clip properties from the file reader
 		if (end == 0.0)
-			End(reader->info.duration);
+			ClipBase::End(reader->info.duration);
 	}
 	else
 		// Throw error if reader not initialized
@@ -356,6 +369,11 @@ float Clip::End() const
 		return end;
 }
 
+// Override End() position
+void Clip::End(float value) {
+	ClipBase::End(value);
+}
+
 // Create an openshot::Frame object for a specific frame number of this reader.
 std::shared_ptr<Frame> Clip::GetFrame(int64_t frame_number)
 {
@@ -422,6 +440,8 @@ std::shared_ptr<Frame> Clip::GetFrame(std::shared_ptr<openshot::Frame> backgroun
 		// Get time mapped frame number (used to increase speed, change direction, etc...)
 		// TODO: Handle variable # of samples, since this resamples audio for different speeds (only when time curve is set)
 		get_time_mapped_frame(original_frame, new_frame_number);
+		// Return the frame's number so the correct keyframes are applied.
+		original_frame->number = frame_number;
 
 		// Apply local effects to the frame (if any)
 		apply_effects(original_frame);
@@ -466,13 +486,13 @@ std::string Clip::get_file_extension(std::string path)
 }
 
 // Reverse an audio buffer
-void Clip::reverse_buffer(juce::AudioSampleBuffer* buffer)
+void Clip::reverse_buffer(juce::AudioBuffer<float>* buffer)
 {
 	int number_of_samples = buffer->getNumSamples();
 	int channels = buffer->getNumChannels();
 
 	// Reverse array (create new buffer to hold the reversed version)
-	juce::AudioSampleBuffer *reversed = new juce::AudioSampleBuffer(channels, number_of_samples);
+	auto *reversed = new juce::AudioBuffer<float>(channels, number_of_samples);
 	reversed->clear();
 
 	for (int channel = 0; channel < channels; channel++)
@@ -490,7 +510,7 @@ void Clip::reverse_buffer(juce::AudioSampleBuffer* buffer)
 		buffer->addFrom(channel, 0, reversed->getReadPointer(channel), number_of_samples, 1.0f);
 
 	delete reversed;
-	reversed = NULL;
+	reversed = nullptr;
 }
 
 // Adjust the audio and image of a time mapped frame
@@ -504,10 +524,10 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 	// Check for a valid time map curve
 	if (time.GetLength() > 1)
 	{
-		const GenericScopedLock<juce::CriticalSection> lock(getFrameCriticalSection);
+		const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
 
 		// create buffer and resampler
-		juce::AudioSampleBuffer *samples = NULL;
+		juce::AudioBuffer<float> *samples = nullptr;
 		if (!resampler)
 			resampler = new AudioResampler();
 
@@ -527,10 +547,10 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 			if (time.GetRepeatFraction(frame_number).den > 1) {
 				// SLOWING DOWN AUDIO
 				// Resample data, and return new buffer pointer
-				juce::AudioSampleBuffer *resampled_buffer = NULL;
+				juce::AudioBuffer<float> *resampled_buffer = nullptr;
 
 				// SLOW DOWN audio (split audio)
-				samples = new juce::AudioSampleBuffer(channels, number_of_samples);
+				samples = new juce::AudioBuffer<float>(channels, number_of_samples);
 				samples->clear();
 
 				// Loop through channels, and get audio samples
@@ -559,7 +579,7 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 										number_of_samples, 1.0f);
 
 				// Clean up
-				resampled_buffer = NULL;
+				resampled_buffer = nullptr;
 
 			}
 			else if (abs(delta) > 1 && abs(delta) < 100) {
@@ -574,7 +594,7 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 																		 reader->info.channels);
 
 					// Allocate a new sample buffer for these delta frames
-					samples = new juce::AudioSampleBuffer(channels, total_delta_samples);
+					samples = new juce::AudioBuffer<float>(channels, total_delta_samples);
 					samples->clear();
 
 					// Loop through each frame in this delta
@@ -582,8 +602,8 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 						 delta_frame <= new_frame_number; delta_frame++) {
 						// buffer to hold detal samples
 						int number_of_delta_samples = GetOrCreateFrame(delta_frame)->GetAudioSamplesCount();
-						juce::AudioSampleBuffer *delta_samples = new juce::AudioSampleBuffer(channels,
-																					   number_of_delta_samples);
+						auto *delta_samples = new juce::AudioBuffer<float>(channels,
+						                                                   number_of_delta_samples);
 						delta_samples->clear();
 
 						for (int channel = 0; channel < channels; channel++)
@@ -602,7 +622,7 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 
 						// Clean up
 						delete delta_samples;
-						delta_samples = NULL;
+						delta_samples = nullptr;
 
 						// Increment start position
 						start += number_of_delta_samples;
@@ -618,7 +638,7 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 																		 reader->info.channels);
 
 					// Allocate a new sample buffer for these delta frames
-					samples = new juce::AudioSampleBuffer(channels, total_delta_samples);
+					samples = new juce::AudioBuffer<float>(channels, total_delta_samples);
 					samples->clear();
 
 					// Loop through each frame in this delta
@@ -626,8 +646,8 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 						 delta_frame >= new_frame_number; delta_frame--) {
 						// buffer to hold delta samples
 						int number_of_delta_samples = GetOrCreateFrame(delta_frame)->GetAudioSamplesCount();
-						juce::AudioSampleBuffer *delta_samples = new juce::AudioSampleBuffer(channels,
-																					   number_of_delta_samples);
+						auto *delta_samples = new juce::AudioBuffer<float>(channels,
+						                                                   number_of_delta_samples);
 						delta_samples->clear();
 
 						for (int channel = 0; channel < channels; channel++)
@@ -657,7 +677,7 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 				resampler->SetBuffer(samples, float(start) / float(number_of_samples));
 
 				// Resample data, and return new buffer pointer
-				juce::AudioSampleBuffer *buffer = resampler->GetResampledBuffer();
+				juce::AudioBuffer<float> *buffer = resampler->GetResampledBuffer();
 
 				// Add the newly resized audio samples to the current frame
 				for (int channel = 0; channel < channels; channel++)
@@ -665,11 +685,11 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 					frame->AddAudio(true, channel, 0, buffer->getReadPointer(channel), number_of_samples, 1.0f);
 
 				// Clean up
-				buffer = NULL;
+				buffer = nullptr;
 			}
 			else {
 				// Use the samples on this frame (but maybe reverse them if needed)
-				samples = new juce::AudioSampleBuffer(channels, number_of_samples);
+				samples = new juce::AudioBuffer<float>(channels, number_of_samples);
 				samples->clear();
 
 				// Loop through channels, and get audio samples
@@ -689,7 +709,7 @@ void Clip::get_time_mapped_frame(std::shared_ptr<Frame> frame, int64_t frame_num
 			}
 
 			delete samples;
-			samples = NULL;
+			samples = nullptr;
 		}
 	}
 }
@@ -710,7 +730,9 @@ std::shared_ptr<Frame> Clip::GetOrCreateFrame(int64_t number)
 {
 	try {
 		// Debug output
-		ZmqLogger::Instance()->AppendDebugMethod("Clip::GetOrCreateFrame (from reader)", "number", number);
+		ZmqLogger::Instance()->AppendDebugMethod(
+			"Clip::GetOrCreateFrame (from reader)",
+			"number", number);
 
 		// Attempt to get a frame (but this could fail if a reader has just been closed)
 		auto reader_frame = reader->GetFrame(number);
@@ -721,8 +743,14 @@ std::shared_ptr<Frame> Clip::GetOrCreateFrame(int64_t number)
 			// This allows a clip to modify the pixels and audio of this frame without
 			// changing the underlying reader's frame data
 			auto reader_copy = std::make_shared<Frame>(*reader_frame.get());
-			reader_copy->SampleRate(reader_frame->SampleRate());
-			reader_copy->ChannelsLayout(reader_frame->ChannelsLayout());
+            if (has_video.GetInt(number) == 0) {
+                // No video, so add transparent pixels
+                reader_copy->AddColor(QColor(Qt::transparent));
+            }
+            if (has_audio.GetInt(number) == 0 || number > reader->info.video_length) {
+                // No audio, so include silence (also, mute audio if past end of reader)
+                reader_copy->AddAudioSilence(reader_copy->GetAudioSamplesCount());
+            }
 			return reader_copy;
 		}
 
@@ -736,7 +764,10 @@ std::shared_ptr<Frame> Clip::GetOrCreateFrame(int64_t number)
 	int estimated_samples_in_frame = Frame::GetSamplesPerFrame(number, reader->info.fps, reader->info.sample_rate, reader->info.channels);
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("Clip::GetOrCreateFrame (create blank)", "number", number, "estimated_samples_in_frame", estimated_samples_in_frame);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"Clip::GetOrCreateFrame (create blank)",
+		"number", number,
+		"estimated_samples_in_frame", estimated_samples_in_frame);
 
 	// Create blank frame
 	auto new_frame = std::make_shared<Frame>(
@@ -807,10 +838,10 @@ std::string Clip::PropertiesJSON(int64_t requested_frame) const {
 	// Add waveform choices (dropdown style)
 	root["waveform"]["choices"].append(add_property_choice_json("Yes", true, waveform));
 	root["waveform"]["choices"].append(add_property_choice_json("No", false, waveform));
-	
+
 	// Add the parentTrackedObject's properties
 	if (parentTrackedObject)
-	{	
+	{
 		// Convert Clip's frame position to Timeline's frame position
 		long clip_start_position = round(Position() * info.fps.ToDouble()) + 1;
 		long clip_start_frame = (Start() * info.fps.ToDouble()) + 1;
@@ -821,7 +852,7 @@ std::string Clip::PropertiesJSON(int64_t requested_frame) const {
 		double parentObject_frame_number = trackedObjectParentClipProperties["frame_number"];
 		// Get attached object properties
 		std::map< std::string, float > trackedObjectProperties = parentTrackedObject->GetBoxValues(parentObject_frame_number);
-		
+
 		// Correct the parent Tracked Object properties by the clip's reference system
 		float parentObject_location_x = trackedObjectProperties["cx"] - 0.5 + trackedObjectParentClipProperties["cx"];
 		float parentObject_location_y = trackedObjectProperties["cy"] - 0.5 + trackedObjectParentClipProperties["cy"];
@@ -847,7 +878,7 @@ std::string Clip::PropertiesJSON(int64_t requested_frame) const {
 		double timeline_frame_number = requested_frame + clip_start_position - clip_start_frame;
 
 		// Correct the parent Clip Object properties by the clip's reference system
-		float parentObject_location_x = parentClipObject->location_x.GetValue(timeline_frame_number); 
+		float parentObject_location_x = parentClipObject->location_x.GetValue(timeline_frame_number);
 		float parentObject_location_y = parentClipObject->location_y.GetValue(timeline_frame_number);
 		float parentObject_scale_x = parentClipObject->scale_x.GetValue(timeline_frame_number);
 		float parentObject_scale_y = parentClipObject->scale_y.GetValue(timeline_frame_number);
@@ -1069,7 +1100,7 @@ void Clip::SetJsonValue(const Json::Value root) {
 
 				// Create instance of effect
 				if ( (e = EffectInfo().CreateEffect(existing_effect["type"].asString()))) {
-					
+
 					// Load Json into Effect
 					e->SetJsonValue(existing_effect);
 
@@ -1090,10 +1121,8 @@ void Clip::SetJsonValue(const Json::Value root) {
 				// Track if reader was open
 				already_open = reader->IsOpen();
 
-				// Close and delete existing reader (if any)
-				reader->Close();
-				delete reader;
-				reader = NULL;
+				// Close and delete existing allocated reader (if any)
+                Reader(NULL);
 			}
 
 			// Create new reader (and load properties)
@@ -1194,7 +1223,7 @@ void Clip::AddEffect(EffectBase* effect)
 
 			// Iterate through effect's vector of Tracked Objects
 			for (auto const& trackedObject : effect->trackedObjects){
-				
+
 				// Cast the Tracked Object as TrackedObjectBBox
 				std::shared_ptr<TrackedObjectBBox> trackedObjectBBox = std::static_pointer_cast<TrackedObjectBBox>(trackedObject.second);
 
@@ -1203,13 +1232,10 @@ void Clip::AddEffect(EffectBase* effect)
 
 				// Add the Tracked Object to the timeline
 				parentTimeline->AddTrackedObject(trackedObjectBBox);
-			}	
+			}
 		}
 	}
     #endif
-
-	// Clear cache
-	cache.Clear();
 }
 
 // Remove an effect from the clip
@@ -1239,10 +1265,10 @@ bool Clip::isEqual(double a, double b)
 // Apply keyframes to the source frame (if any)
 void Clip::apply_keyframes(std::shared_ptr<Frame> frame, std::shared_ptr<QImage> background_canvas) {
     // Skip out if video was disabled or only an audio frame (no visualisation in use)
-    if (has_video.GetInt(frame->number) == 0 ||
-        (!Waveform() && !Reader()->info.has_video))
+    if (!Waveform() && !Reader()->info.has_video) {
         // Skip the rest of the image processing for performance reasons
         return;
+    }
 
     // Get image from clip
     std::shared_ptr<QImage> source_image = frame->GetImage();
@@ -1251,7 +1277,12 @@ void Clip::apply_keyframes(std::shared_ptr<Frame> frame, std::shared_ptr<QImage>
     if (Waveform())
     {
         // Debug output
-        ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Generate Waveform Image)", "frame->number", frame->number, "Waveform()", Waveform());
+        ZmqLogger::Instance()->AppendDebugMethod(
+            "Clip::get_transform (Generate Waveform Image)",
+            "frame->number", frame->number,
+            "Waveform()", Waveform(),
+            "background_canvas->width()", background_canvas->width(),
+            "background_canvas->height()", background_canvas->height());
 
         // Get the color of the waveform
         int red = wave_color.red.GetInt(frame->number);
@@ -1261,17 +1292,18 @@ void Clip::apply_keyframes(std::shared_ptr<Frame> frame, std::shared_ptr<QImage>
 
         // Generate Waveform Dynamically (the size of the timeline)
         source_image = frame->GetWaveform(background_canvas->width(), background_canvas->height(), red, green, blue, alpha);
+        frame->AddImage(source_image);
     }
 
-    // Size of final image
-    int width = background_canvas->width();
-    int height = background_canvas->height();
-
     // Get transform from clip's keyframes
-    QTransform transform = get_transform(frame, width, height);
+    QTransform transform = get_transform(frame, background_canvas->width(), background_canvas->height());
 
     // Debug output
-    ZmqLogger::Instance()->AppendDebugMethod("Clip::ApplyKeyframes (Transform: Composite Image Layer: Prepare)", "frame->number", frame->number);
+    ZmqLogger::Instance()->AppendDebugMethod(
+        "Clip::ApplyKeyframes (Transform: Composite Image Layer: Prepare)",
+        "frame->number", frame->number,
+        "background_canvas->width()", background_canvas->width(),
+        "background_canvas->height()", background_canvas->height());
 
     // Load timeline's new frame image into a QPainter
     QPainter painter(background_canvas.get());
@@ -1300,11 +1332,11 @@ void Clip::apply_keyframes(std::shared_ptr<Frame> frame, std::shared_ptr<QImage>
                     break;
 
                 case (FRAME_DISPLAY_TIMELINE):
-                    frame_number_str << (position * t->info.fps.ToFloat()) + frame->number;
+                    frame_number_str << round((Position() - Start()) * t->info.fps.ToFloat()) + frame->number;
                     break;
 
                 case (FRAME_DISPLAY_BOTH):
-                    frame_number_str << (position * t->info.fps.ToFloat()) + frame->number << " (" << frame->number << ")";
+                    frame_number_str << round((Position() - Start()) * t->info.fps.ToFloat()) + frame->number << " (" << frame->number << ")";
                     break;
             }
 
@@ -1325,7 +1357,7 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
     // Get image from clip
     std::shared_ptr<QImage> source_image = frame->GetImage();
 
-	/* ALPHA & OPACITY */
+    /* ALPHA & OPACITY */
 	if (alpha.GetValue(frame->number) != 1.0)
 	{
 		float alpha_value = alpha.GetValue(frame->number);
@@ -1345,13 +1377,16 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 		}
 
 		// Debug output
-		ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Set Alpha & Opacity)", "alpha_value", alpha_value, "frame->number", frame->number);
+		ZmqLogger::Instance()->AppendDebugMethod(
+			"Clip::get_transform (Set Alpha & Opacity)",
+			"alpha_value", alpha_value,
+			"frame->number", frame->number);
 	}
 
 	/* RESIZE SOURCE IMAGE - based on scale type */
 	QSize source_size = source_image->size();
 
-	// Apply stretch scale to correctly fit the bounding-box
+    // Apply stretch scale to correctly fit the bounding-box
 	if (parentTrackedObject){
 		scale = SCALE_STRETCH;
 	}
@@ -1362,29 +1397,45 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 			source_size.scale(width, height, Qt::KeepAspectRatio);
 
 			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Scale: SCALE_FIT)", "frame->number", frame->number, "source_width", source_size.width(), "source_height", source_size.height());
+			ZmqLogger::Instance()->AppendDebugMethod(
+				"Clip::get_transform (Scale: SCALE_FIT)",
+				"frame->number", frame->number,
+				"source_width", source_size.width(),
+				"source_height", source_size.height());
 			break;
 		}
 		case (SCALE_STRETCH): {
 			source_size.scale(width, height, Qt::IgnoreAspectRatio);
 
 			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Scale: SCALE_STRETCH)", "frame->number", frame->number, "source_width", source_size.width(), "source_height", source_size.height());
+			ZmqLogger::Instance()->AppendDebugMethod(
+				"Clip::get_transform (Scale: SCALE_STRETCH)",
+				"frame->number", frame->number,
+				"source_width", source_size.width(),
+				"source_height", source_size.height());
 			break;
 		}
 		case (SCALE_CROP): {
 			source_size.scale(width, height, Qt::KeepAspectRatioByExpanding);
 
 			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Scale: SCALE_CROP)", "frame->number", frame->number, "source_width", source_size.width(), "source_height", source_size.height());
+			ZmqLogger::Instance()->AppendDebugMethod(
+				"Clip::get_transform (Scale: SCALE_CROP)",
+				"frame->number", frame->number,
+				"source_width", source_size.width(),
+				"source_height", source_size.height());
 			break;
 		}
 		case (SCALE_NONE): {
-		    // Image is already the original size (i.e. no scaling mode) relative
-		    // to the preview window size (i.e. timeline / preview ratio). No further
-		    // scaling is needed here.
+			// Image is already the original size (i.e. no scaling mode) relative
+			// to the preview window size (i.e. timeline / preview ratio). No further
+			// scaling is needed here.
 			// Debug output
-			ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Scale: SCALE_NONE)", "frame->number", frame->number, "source_width", source_size.width(), "source_height", source_size.height());
+			ZmqLogger::Instance()->AppendDebugMethod(
+				"Clip::get_transform (Scale: SCALE_NONE)",
+				"frame->number", frame->number,
+				"source_width", source_size.width(),
+				"source_height", source_size.height());
 			break;
 		}
 	}
@@ -1407,7 +1458,7 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 		double timeline_frame_number = frame->number + clip_start_position - clip_start_frame;
 
 		// Get parent object's properties (Clip)
-		parentObject_location_x = parentClipObject->location_x.GetValue(timeline_frame_number); 
+		parentObject_location_x = parentClipObject->location_x.GetValue(timeline_frame_number);
 		parentObject_location_y = parentClipObject->location_y.GetValue(timeline_frame_number);
 		parentObject_scale_x = parentClipObject->scale_x.GetValue(timeline_frame_number);
 		parentObject_scale_y = parentClipObject->scale_y.GetValue(timeline_frame_number);
@@ -1442,8 +1493,8 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 			parentObject_scale_x = trackedObjectProperties["w"]*trackedObjectProperties["sx"];
 			parentObject_scale_y = trackedObjectProperties["h"]*trackedObjectProperties["sy"];
 			parentObject_rotation = trackedObjectProperties["r"] + trackedObjectParentClipProperties["rotation"];
-		} 
-		else 
+		}
+		else
 		{
 			// Access the parentTrackedObject's properties
 			std::map<std::string, float> trackedObjectProperties = parentTrackedObject->GetBoxValues(timeline_frame_number);
@@ -1471,9 +1522,9 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 		sy*= parentObject_scale_y;
 	}
 
-	float scaled_source_width = source_size.width() * sx;
+    float scaled_source_width = source_size.width() * sx;
 	float scaled_source_height = source_size.height() * sy;
-	
+
 	switch (gravity)
 	{
 		case (GRAVITY_TOP_LEFT):
@@ -1510,10 +1561,15 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 	}
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Gravity)", "frame->number", frame->number, "source_clip->gravity", gravity, "scaled_source_width", scaled_source_width, "scaled_source_height", scaled_source_height);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"Clip::get_transform (Gravity)",
+		"frame->number", frame->number,
+		"source_clip->gravity", gravity,
+		"scaled_source_width", scaled_source_width,
+		"scaled_source_height", scaled_source_height);
 
 	QTransform transform;
-		
+
 	/* LOCATION, ROTATION, AND SCALE */
 	float r = rotation.GetValue(frame->number) + parentObject_rotation; // rotate in degrees
 	x += (width * (location_x.GetValue(frame->number) + parentObject_location_x )); // move in percentage of final width
@@ -1524,12 +1580,17 @@ QTransform Clip::get_transform(std::shared_ptr<Frame> frame, int width, int heig
 	float origin_y_value = origin_y.GetValue(frame->number);
 
 	// Transform source image (if needed)
-	ZmqLogger::Instance()->AppendDebugMethod("Clip::get_transform (Build QTransform - if needed)", "frame->number", frame->number, "x", x, "y", y, "r", r, "sx", sx, "sy", sy);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"Clip::get_transform (Build QTransform - if needed)",
+		"frame->number", frame->number,
+		"x", x, "y", y,
+		"r", r,
+		"sx", sx, "sy", sy);
 
 	if (!isEqual(x, 0) || !isEqual(y, 0)) {
 		// TRANSLATE/MOVE CLIP
 		transform.translate(x, y);
-	} 
+	}
 	if (!isEqual(r, 0) || !isEqual(shear_x_value, 0) || !isEqual(shear_y_value, 0)) {
 		// ROTATE CLIP (around origin_x, origin_y)
 		float origin_x_offset = (scaled_source_width * origin_x_value);
