@@ -6,37 +6,24 @@
  * @ref License
  */
 
-/* LICENSE
- *
- * Copyright (c) 2008-2019 OpenShot Studios, LLC
- * <http://www.openshotstudios.com/>. This file is part of
- * OpenShot Library (libopenshot), an open-source project dedicated to
- * delivering high quality video editing and animation solutions to the
- * world. For more information visit <http://www.openshot.org/>.
- *
- * OpenShot Library (libopenshot) is free software: you can redistribute it
- * and/or modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * OpenShot Library (libopenshot) is distributed in the hope that it will be
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with OpenShot Library. If not, see <http://www.gnu.org/licenses/>.
- */
+// Copyright (c) 2008-2019 OpenShot Studios, LLC
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
+#include <cmath>
+#include <iostream>
+#include <iomanip>
 
 #include "FrameMapper.h"
 #include "Exceptions.h"
 #include "Clip.h"
+#include "ZmqLogger.h"
 
 using namespace std;
 using namespace openshot;
 
 FrameMapper::FrameMapper(ReaderBase *reader, Fraction target, PulldownType target_pulldown, int target_sample_rate, int target_channels, ChannelLayout target_channel_layout) :
-		reader(reader), target(target), pulldown(target_pulldown), is_dirty(true), avr(NULL), parent_position(0.0)
+		reader(reader), target(target), pulldown(target_pulldown), is_dirty(true), avr(NULL), parent_position(0.0), parent_start(0.0)
 {
 	// Set the original frame rate from the reader
 	original = Fraction(reader->info.fps.num, reader->info.fps.den);
@@ -58,7 +45,7 @@ FrameMapper::FrameMapper(ReaderBase *reader, Fraction target, PulldownType targe
 	field_toggle = true;
 
 	// Adjust cache size based on size of frame and audio
-	final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
+	final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS, info.width, info.height, info.sample_rate, info.channels);
 }
 
 // Destructor
@@ -95,6 +82,18 @@ void FrameMapper::AddField(Field field)
 	field_toggle = (field_toggle ? false : true);
 }
 
+// Clear both the fields & frames lists
+void FrameMapper::Clear() {
+    // Prevent async calls to the following code
+    const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
+
+    // Clear the fields & frames lists
+    fields.clear();
+    fields.shrink_to_fit();
+    frames.clear();
+    frames.shrink_to_fit();
+}
+
 // Use the original and target frame rates and a pull-down technique to create
 // a mapping between the original fields and frames or a video to a new frame rate.
 // This might repeat or skip fields and frames of the original video, depending on
@@ -108,9 +107,11 @@ void FrameMapper::Init()
 		// Skip initialization
 		return;
 
+	// Prevent async calls to the following code
+	const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
+
 	// Clear the fields & frames lists
-	fields.clear();
-	frames.clear();
+	Clear();
 
 	// Find parent position (if any)
     Clip *parent = (Clip *) ParentClip();
@@ -224,7 +225,7 @@ void FrameMapper::Init()
 		int64_t new_length = reader->info.video_length * rate_diff;
 
 		// Calculate the value difference
-		double value_increment = (reader->info.video_length + 1) / (double) (new_length);
+		double value_increment = reader->info.video_length / (double) (new_length);
 
 		// Loop through curve, and build list of frames
 		double original_frame_num = 1.0f;
@@ -322,6 +323,7 @@ void FrameMapper::Init()
 
 	// Clear the internal fields list (no longer needed)
 	fields.clear();
+	fields.shrink_to_fit();
 }
 
 MappedFrame FrameMapper::GetMappedFrame(int64_t TargetFrameNumber)
@@ -355,7 +357,12 @@ MappedFrame FrameMapper::GetMappedFrame(int64_t TargetFrameNumber)
 		TargetFrameNumber = frames.size();
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetMappedFrame", "TargetFrameNumber", TargetFrameNumber, "frames.size()", frames.size(), "frames[...].Odd", frames[TargetFrameNumber - 1].Odd.Frame, "frames[...].Even", frames[TargetFrameNumber - 1].Even.Frame);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::GetMappedFrame",
+		"TargetFrameNumber", TargetFrameNumber,
+		"frames.size()", frames.size(),
+		"frames[...].Odd", frames[TargetFrameNumber - 1].Odd.Frame,
+		"frames[...].Even", frames[TargetFrameNumber - 1].Even.Frame);
 
 	// Return frame
 	return frames[TargetFrameNumber - 1];
@@ -371,7 +378,10 @@ std::shared_ptr<Frame> FrameMapper::GetOrCreateFrame(int64_t number)
 
 	try {
 		// Debug output
-		ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetOrCreateFrame (from reader)", "number", number, "samples_in_frame", samples_in_frame);
+		ZmqLogger::Instance()->AppendDebugMethod(
+			"FrameMapper::GetOrCreateFrame (from reader)",
+			"number", number,
+			"samples_in_frame", samples_in_frame);
 
 		// Attempt to get a frame (but this could fail if a reader has just been closed)
 		new_frame = reader->GetFrame(number);
@@ -386,7 +396,10 @@ std::shared_ptr<Frame> FrameMapper::GetOrCreateFrame(int64_t number)
 	}
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetOrCreateFrame (create blank)", "number", number, "samples_in_frame", samples_in_frame);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::GetOrCreateFrame (create blank)",
+		"number", number,
+		"samples_in_frame", samples_in_frame);
 
 	// Create blank frame
 	new_frame = std::make_shared<Frame>(number, info.width, info.height, "#000000", samples_in_frame, reader->info.channels);
@@ -404,7 +417,7 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 	if (final_frame) return final_frame;
 
 	// Create a scoped lock, allowing only a single thread to run the following code at one time
-	const GenericScopedLock<CriticalSection> lock(getFrameCriticalSection);
+	const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
 
     // Find parent properties (if any)
     Clip *parent = (Clip *) ParentClip();
@@ -431,14 +444,21 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 	int minimum_frames = 1;
 
 	// Debug output
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetFrame (Loop through frames)", "requested_frame", requested_frame, "minimum_frames", minimum_frames);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::GetFrame (Loop through frames)",
+		"requested_frame", requested_frame,
+		"minimum_frames", minimum_frames);
 
 	// Loop through all requested frames
 	for (int64_t frame_number = requested_frame; frame_number < requested_frame + minimum_frames; frame_number++)
 	{
 
 		// Debug output
-		ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::GetFrame (inside omp for loop)", "frame_number", frame_number, "minimum_frames", minimum_frames, "requested_frame", requested_frame);
+		ZmqLogger::Instance()->AppendDebugMethod(
+			"FrameMapper::GetFrame (inside omp for loop)",
+			"frame_number", frame_number,
+			"minimum_frames", minimum_frames,
+			"requested_frame", requested_frame);
 
 		// Get the mapped frame
 		MappedFrame mapped = GetMappedFrame(frame_number);
@@ -517,7 +537,7 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 			copy_samples.sample_end += EXTRA_INPUT_SAMPLES;
 			int samples_per_end_frame =
 				Frame::GetSamplesPerFrame(copy_samples.frame_end, original,
-										  reader->info.sample_rate, reader->info.channels);
+				                          reader->info.sample_rate, reader->info.channels);
 			if (copy_samples.sample_end >= samples_per_end_frame)
 			{
 				// check for wrapping
@@ -533,7 +553,7 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 				copy_samples.sample_start += EXTRA_INPUT_SAMPLES;
 				int samples_per_start_frame =
 					Frame::GetSamplesPerFrame(copy_samples.frame_start, original,
-											  reader->info.sample_rate, reader->info.channels);
+					                          reader->info.sample_rate, reader->info.channels);
 				if (copy_samples.sample_start >= samples_per_start_frame)
 				{
 					// check for wrapping
@@ -611,7 +631,7 @@ std::shared_ptr<Frame> FrameMapper::GetFrame(int64_t requested_frame)
 	return final_cache.GetFrame(requested_frame);
 }
 
-void FrameMapper::PrintMapping()
+void FrameMapper::PrintMapping(std::ostream* out)
 {
 	// Check if mappings are dirty (and need to be recalculated)
 	if (is_dirty)
@@ -622,8 +642,16 @@ void FrameMapper::PrintMapping()
 	for (float map = 1; map <= frames.size(); map++)
 	{
 		MappedFrame frame = frames[map - 1];
-		cout << "Target frame #: " << map << " mapped to original frame #:\t(" << frame.Odd.Frame << " odd, " << frame.Even.Frame << " even)" << endl;
-		cout << "  - Audio samples mapped to frame " << frame.Samples.frame_start << ":" << frame.Samples.sample_start << " to frame " << frame.Samples.frame_end << ":" << frame.Samples.sample_end << endl;
+		*out << "Target frame #: " << map
+		     << " mapped to original frame #:\t("
+		     << frame.Odd.Frame << " odd, "
+		     << frame.Even.Frame << " even)" << std::endl;
+
+		*out << "  - Audio samples mapped to frame "
+		     << frame.Samples.frame_start << ":"
+		     << frame.Samples.sample_start << " to frame "
+		     << frame.Samples.frame_end << ":"
+		     << frame.Samples.sample_end << endl;
 	}
 
 }
@@ -654,30 +682,29 @@ void FrameMapper::Close()
 	if (reader)
 	{
 		// Create a scoped lock, allowing only a single thread to run the following code at one time
-		const GenericScopedLock<CriticalSection> lock(getFrameCriticalSection);
+		const std::lock_guard<std::recursive_mutex> lock(getFrameMutex);
 
 		ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::Close");
 
 		// Close internal reader
 		reader->Close();
-
-		// Clear the fields & frames lists
-		fields.clear();
-		frames.clear();
-
-		// Mark as dirty
-		is_dirty = true;
-
-		// Clear cache
-		final_cache.Clear();
-
-		// Deallocate resample buffer
-		if (avr) {
-			SWR_CLOSE(avr);
-			SWR_FREE(&avr);
-			avr = NULL;
-		}
 	}
+
+    // Clear the fields & frames lists
+    Clear();
+
+    // Mark as dirty
+    is_dirty = true;
+
+    // Clear cache
+    final_cache.Clear();
+
+    // Deallocate resample buffer
+    if (avr) {
+        SWR_CLOSE(avr);
+        SWR_FREE(&avr);
+        avr = NULL;
+    }
 }
 
 
@@ -733,7 +760,14 @@ void FrameMapper::SetJsonValue(const Json::Value root) {
 // Change frame rate or audio mapping details
 void FrameMapper::ChangeMapping(Fraction target_fps, PulldownType target_pulldown,  int target_sample_rate, int target_channels, ChannelLayout target_channel_layout)
 {
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ChangeMapping", "target_fps.num", target_fps.num, "target_fps.den", target_fps.den, "target_pulldown", target_pulldown, "target_sample_rate", target_sample_rate, "target_channels", target_channels, "target_channel_layout", target_channel_layout);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::ChangeMapping",
+		"target_fps.num", target_fps.num,
+		"target_fps.den", target_fps.den,
+		"target_pulldown", target_pulldown,
+		"target_sample_rate", target_sample_rate,
+		"target_channels", target_channels,
+		"target_channel_layout", target_channel_layout);
 
 	// Mark as dirty
 	is_dirty = true;
@@ -745,6 +779,7 @@ void FrameMapper::ChangeMapping(Fraction target_fps, PulldownType target_pulldow
 	info.fps.den = target_fps.den;
 	info.video_timebase.num = target_fps.den;
 	info.video_timebase.den = target_fps.num;
+	info.video_length = round(info.duration * info.fps.ToDouble());
 	pulldown = target_pulldown;
 	info.sample_rate = target_sample_rate;
 	info.channels = target_channels;
@@ -754,7 +789,7 @@ void FrameMapper::ChangeMapping(Fraction target_fps, PulldownType target_pulldow
 	final_cache.Clear();
 
 	// Adjust cache size based on size of frame and audio
-	final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS * 2, info.width, info.height, info.sample_rate, info.channels);
+	final_cache.SetMaxBytesFromInfo(OPEN_MP_NUM_PROCESSORS, info.width, info.height, info.sample_rate, info.channels);
 
 	// Deallocate resample buffer
 	if (avr) {
@@ -779,7 +814,13 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	int samples_in_frame = frame->GetAudioSamplesCount();
 	ChannelLayout channel_layout_in_frame = frame->ChannelsLayout();
 
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio", "frame->number", frame->number, "original_frame_number", original_frame_number, "channels_in_frame", channels_in_frame, "samples_in_frame", samples_in_frame, "sample_rate_in_frame", sample_rate_in_frame);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::ResampleMappedAudio",
+		"frame->number", frame->number,
+		"original_frame_number", original_frame_number,
+		"channels_in_frame", channels_in_frame,
+		"samples_in_frame", samples_in_frame,
+		"sample_rate_in_frame", sample_rate_in_frame);
 
 	// Get audio sample array
 	float* frame_samples_float = NULL;
@@ -815,7 +856,14 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	delete[] frame_samples_float;
 	frame_samples_float = NULL;
 
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (got sample data from frame)", "frame->number", frame->number, "total_frame_samples", total_frame_samples, "target channels", info.channels, "channels_in_frame", channels_in_frame, "target sample_rate", info.sample_rate, "samples_in_frame", samples_in_frame);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::ResampleMappedAudio (got sample data from frame)",
+		"frame->number", frame->number,
+		"total_frame_samples", total_frame_samples,
+		"target channels", info.channels,
+		"channels_in_frame", channels_in_frame,
+		"target sample_rate", info.sample_rate,
+		"samples_in_frame", samples_in_frame);
 
 
 	// Create input frame (and allocate arrays)
@@ -823,8 +871,10 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	AV_RESET_FRAME(audio_frame);
 	audio_frame->nb_samples = total_frame_samples / channels_in_frame;
 
-	int error_code = avcodec_fill_audio_frame(audio_frame, channels_in_frame, AV_SAMPLE_FMT_S16, (uint8_t *) frame_samples,
-			audio_frame->nb_samples * av_get_bytes_per_sample(AV_SAMPLE_FMT_S16) * channels_in_frame, 1);
+	int buf_size = audio_frame->nb_samples * av_get_bytes_per_sample(AV_SAMPLE_FMT_S16) * channels_in_frame;
+	int error_code = avcodec_fill_audio_frame(
+		audio_frame, channels_in_frame, AV_SAMPLE_FMT_S16,
+		(uint8_t *) frame_samples, buf_size, 1);
 
 	if (error_code < 0)
 	{
@@ -837,7 +887,14 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	// Update total samples & input frame size (due to bigger or smaller data types)
 	total_frame_samples = Frame::GetSamplesPerFrame(AdjustFrameNumber(frame->number), target, info.sample_rate, info.channels);
 
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (adjust # of samples)", "total_frame_samples", total_frame_samples, "info.sample_rate", info.sample_rate, "sample_rate_in_frame", sample_rate_in_frame, "info.channels", info.channels, "channels_in_frame", channels_in_frame, "original_frame_number", original_frame_number);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::ResampleMappedAudio (adjust # of samples)",
+		"total_frame_samples", total_frame_samples,
+		"info.sample_rate", info.sample_rate,
+		"sample_rate_in_frame", sample_rate_in_frame,
+		"info.channels", info.channels,
+		"channels_in_frame", channels_in_frame,
+		"original_frame_number", original_frame_number);
 
 	// Create output frame (and allocate arrays)
 	AVFrame *audio_converted = AV_ALLOCATE_FRAME();
@@ -845,32 +902,39 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	audio_converted->nb_samples = total_frame_samples;
 	av_samples_alloc(audio_converted->data, audio_converted->linesize, info.channels, total_frame_samples, AV_SAMPLE_FMT_S16, 0);
 
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (preparing for resample)", "in_sample_fmt", AV_SAMPLE_FMT_S16, "out_sample_fmt", AV_SAMPLE_FMT_S16, "in_sample_rate", sample_rate_in_frame, "out_sample_rate", info.sample_rate, "in_channels", channels_in_frame, "out_channels", info.channels);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::ResampleMappedAudio (preparing for resample)",
+		"in_sample_fmt", AV_SAMPLE_FMT_S16,
+		"out_sample_fmt", AV_SAMPLE_FMT_S16,
+		"in_sample_rate", sample_rate_in_frame,
+		"out_sample_rate", info.sample_rate,
+		"in_channels", channels_in_frame,
+		"out_channels", info.channels);
 
 	int nb_samples = 0;
 
     // setup resample context
     if (!avr) {
         avr = SWR_ALLOC();
-        av_opt_set_int(avr,  "in_channel_layout", channel_layout_in_frame, 0);
-        av_opt_set_int(avr, "out_channel_layout", info.channel_layout, 0);
-        av_opt_set_int(avr,  "in_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
-        av_opt_set_int(avr, "out_sample_fmt",     AV_SAMPLE_FMT_S16,     0);
-        av_opt_set_int(avr,  "in_sample_rate",    sample_rate_in_frame,    0);
-        av_opt_set_int(avr, "out_sample_rate",    info.sample_rate,    0);
-        av_opt_set_int(avr,  "in_channels",       channels_in_frame,    0);
-        av_opt_set_int(avr, "out_channels",       info.channels,    0);
+        av_opt_set_int(avr, "in_channel_layout",  channel_layout_in_frame, 0);
+        av_opt_set_int(avr, "out_channel_layout", info.channel_layout,     0);
+        av_opt_set_int(avr, "in_sample_fmt",      AV_SAMPLE_FMT_S16,       0);
+        av_opt_set_int(avr, "out_sample_fmt",     AV_SAMPLE_FMT_S16,       0);
+        av_opt_set_int(avr, "in_sample_rate",     sample_rate_in_frame,    0);
+        av_opt_set_int(avr, "out_sample_rate",    info.sample_rate,        0);
+        av_opt_set_int(avr, "in_channels",        channels_in_frame,       0);
+        av_opt_set_int(avr, "out_channels",       info.channels,           0);
         SWR_INIT(avr);
     }
 
     // Convert audio samples
-    nb_samples = SWR_CONVERT(avr, 	// audio resample context
-            audio_converted->data, 			// output data pointers
-            audio_converted->linesize[0], 	// output plane size, in bytes. (0 if unknown)
-            audio_converted->nb_samples,	// maximum number of samples that the output buffer can hold
-            audio_frame->data,				// input data pointers
-            audio_frame->linesize[0],		// input plane size, in bytes (0 if unknown)
-            audio_frame->nb_samples);		// number of input samples to convert
+    nb_samples = SWR_CONVERT(avr,      // audio resample context
+        audio_converted->data,         // output data pointers
+        audio_converted->linesize[0],  // output plane size, in bytes. (0 if unknown)
+        audio_converted->nb_samples,   // maximum number of samples that the output buffer can hold
+        audio_frame->data,             // input data pointers
+        audio_frame->linesize[0],      // input plane size, in bytes (0 if unknown)
+        audio_frame->nb_samples);      // number of input samples to convert
 
 	// Create a new array (to hold all resampled S16 audio samples)
 	int16_t* resampled_samples = new int16_t[(nb_samples * info.channels)];
@@ -889,7 +953,14 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 	int channel_buffer_size = nb_samples;
 	frame->ResizeAudio(info.channels, channel_buffer_size, info.sample_rate, info.channel_layout);
 
-	ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (Audio successfully resampled)", "nb_samples", nb_samples, "total_frame_samples", total_frame_samples, "info.sample_rate", info.sample_rate, "channels_in_frame", channels_in_frame, "info.channels", info.channels, "info.channel_layout", info.channel_layout);
+	ZmqLogger::Instance()->AppendDebugMethod(
+		"FrameMapper::ResampleMappedAudio (Audio successfully resampled)",
+		"nb_samples", nb_samples,
+		"total_frame_samples", total_frame_samples,
+		"info.sample_rate", info.sample_rate,
+		"channels_in_frame", channels_in_frame,
+		"info.channels", info.channels,
+		"info.channel_layout", info.channel_layout);
 
 	// Array of floats (to hold samples for each channel)
 	float *channel_buffer = new float[channel_buffer_size];
@@ -929,7 +1000,10 @@ void FrameMapper::ResampleMappedAudio(std::shared_ptr<Frame> frame, int64_t orig
 		// Add samples to frame for this channel
 		frame->AddAudio(true, channel_filter, 0, channel_buffer, position, 1.0f);
 
-		ZmqLogger::Instance()->AppendDebugMethod("FrameMapper::ResampleMappedAudio (Add audio to channel)", "number of samples", position, "channel_filter", channel_filter);
+		ZmqLogger::Instance()->AppendDebugMethod(
+			"FrameMapper::ResampleMappedAudio (Add audio to channel)",
+			"number of samples", position,
+			"channel_filter", channel_filter);
 	}
 
 	// Update frame's audio meta data
@@ -957,9 +1031,10 @@ int64_t FrameMapper::AdjustFrameNumber(int64_t clip_frame_number) {
 		start = parent->Start();
 	}
 
-	// Adjust start frame and position based on parent clip. This prevents ensures the same
-	// frame # is used by mapped readers and clips, when calculating samples per frame. Thus,
-	// this prevents gaps and mismatches in # of samples.
+	// Adjust start frame and position based on parent clip.
+	// This ensures the same frame # is used by mapped readers and clips,
+	// when calculating samples per frame.
+	// Thus, this prevents gaps and mismatches in # of samples.
 	int64_t clip_start_frame = (start * info.fps.ToDouble()) + 1;
 	int64_t clip_start_position = round(position * info.fps.ToDouble()) + 1;
 	int64_t frame_number = clip_frame_number + clip_start_position - clip_start_frame;
