@@ -6,33 +6,16 @@
  * @ref License
  */
 
-/* LICENSE
- *
- * Copyright (c) 2008-2019 OpenShot Studios, LLC
- * <http://www.openshotstudios.com/>. This file is part of
- * OpenShot Library (libopenshot), an open-source project dedicated to
- * delivering high quality video editing and animation solutions to the
- * world. For more information visit <http://www.openshot.org/>.
- *
- * OpenShot Library (libopenshot) is free software: you can redistribute it
- * and/or modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation, either version 3 of the
- * License, or (at your option) any later version.
- *
- * OpenShot Library (libopenshot) is distributed in the hope that it will be
- * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with OpenShot Library. If not, see <http://www.gnu.org/licenses/>.
- */
+// Copyright (c) 2008-2019 OpenShot Studios, LLC
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
 
-#include <catch2/catch.hpp>
+#include "openshot_catch.h"
 
 #include "CacheMemory.h"
 #include "Clip.h"
 #include "DummyReader.h"
+#include "Exceptions.h"
 #include "FFmpegReader.h"
 #include "Fraction.h"
 #include "Frame.h"
@@ -191,7 +174,7 @@ TEST_CASE( "30_fps_to_24_fps_Pulldown_None", "[libopenshot][framemapper]" )
 
 TEST_CASE( "resample_audio_48000_to_41000", "[libopenshot][framemapper]" )
 {
-	// Create a reader: 24 fps, 2 channels, 48000 sample rate
+	// Create a reader
 	std::stringstream path;
 	path << TEST_MEDIA_PATH << "sintel_trailer-720p.mp4";
 	FFmpegReader r(path.str());
@@ -205,6 +188,7 @@ TEST_CASE( "resample_audio_48000_to_41000", "[libopenshot][framemapper]" )
 	CHECK(map.GetFrame(1)->GetAudioSamplesCount() == 1470);
 	CHECK(map.GetFrame(2)->GetAudioSamplesCount() == 1470);
 	CHECK(map.GetFrame(50)->GetAudioSamplesCount() == 1470);
+	CHECK(map.info.video_length == 1558);
 
 	// Change mapping data
 	map.ChangeMapping(Fraction(25,1), PULLDOWN_NONE, 22050, 1, LAYOUT_MONO);
@@ -214,6 +198,7 @@ TEST_CASE( "resample_audio_48000_to_41000", "[libopenshot][framemapper]" )
 	CHECK(map.GetFrame(1)->GetAudioSamplesCount() == Approx(882).margin(10.0));
 	CHECK(map.GetFrame(2)->GetAudioSamplesCount() == Approx(882).margin(10.0));
 	CHECK(map.GetFrame(50)->GetAudioSamplesCount() == Approx(882).margin(10.0));
+	CHECK(map.info.video_length == 1299);
 
 	// Close mapper
 	map.Close();
@@ -260,7 +245,7 @@ TEST_CASE( "resample_audio_mapper", "[libopenshot][framemapper]" ) {
 		delete[] audio_buffer;
 	}
 
-	// Create a default fraction (should be 1/1)
+	// Create a dummy reader, with a pre-existing cache
 	openshot::DummyReader r(openshot::Fraction(30, 1), 1, 1, 44100, 2, 30.0, &cache);
 	r.Open(); // Open the reader
 
@@ -400,7 +385,7 @@ TEST_CASE( "redistribute_samples_per_frame", "[libopenshot][framemapper]" ) {
 		delete[] audio_buffer;
 	}
 
-	// Create a default fraction (should be 1/1)
+	// Create a dummy reader, with a pre-existing cache
 	openshot::DummyReader r(openshot::Fraction(30, 1), 1920, 1080, 44100, 2, 30.0, &cache);
 	r.Open(); // Open the reader
 
@@ -619,6 +604,41 @@ TEST_CASE( "Distribute samples", "[libopenshot][framemapper]" ) {
     } // for rates
 }
 
+TEST_CASE( "PrintMapping", "[libopenshot][framemapper]" )
+{
+	const std::string expected(
+		R"(Target frame #: 1 mapped to original frame #:	(1 odd, 1 even)
+  - Audio samples mapped to frame 1:0 to frame 1:1599
+Target frame #: 2 mapped to original frame #:	(2 odd, 2 even)
+  - Audio samples mapped to frame 1:1600 to frame 2:1199
+Target frame #: 3 mapped to original frame #:	(2 odd, 3 even)
+  - Audio samples mapped to frame 2:1200 to frame 3:799
+Target frame #: 4 mapped to original frame #:	(3 odd, 4 even)
+  - Audio samples mapped to frame 3:800 to frame 4:399
+Target frame #: 5 mapped to original frame #:	(4 odd, 4 even)
+  - Audio samples mapped to frame 4:400 to frame 4:1999
+Target frame #: 6 mapped to original frame #:	(5 odd, 5 even)
+  - Audio samples mapped to frame 5:0 to frame 5:1599
+Target frame #: 7 mapped to original frame #:	(6 odd, 6 even)
+  - Audio samples mapped to frame 5:1600 to frame 6:1199
+Target frame #: 8 mapped to original frame #:	(6 odd, 7 even)
+  - Audio samples mapped to frame 6:1200 to frame 7:799
+Target frame #: 9 mapped to original frame #:	(7 odd, 8 even)
+  - Audio samples mapped to frame 7:800 to frame 8:399
+Target frame #: 10 mapped to original frame #:	(8 odd, 8 even)
+  - Audio samples mapped to frame 8:400 to frame 8:1999)");
+
+	DummyReader r(Fraction(24,1), 720, 480, 48000, 2, 5.0);
+	// Create mapping 24 fps and 30 fps
+	FrameMapper mapping(
+		&r, Fraction(30, 1), PULLDOWN_CLASSIC, 48000, 2, LAYOUT_STEREO);
+	std::stringstream mapping_out;
+	mapping.PrintMapping(&mapping_out);
+
+	// Compare a [0, expected.size()) substring of output to expected
+	CHECK(mapping_out.str().substr(0, expected.size()) == expected);
+}
+
 TEST_CASE( "Json", "[libopenshot][framemapper]" )
 {
 	DummyReader r(Fraction(30,1), 1280, 720, 48000, 2, 5.0);
